@@ -73,29 +73,40 @@ test_that("`seed` works (and restores the RNG state afterwards)", {
   }
 })
 
-test_that("`d_test` works", {
+## d_test -----------------------------------------------------------------
+
+test_that(paste(
+  "`d_test` set to the training data gives the same results as its default"
+), {
   skip_if_not(run_vs)
   tstsetups <- names(vss)
-  ### Alternative with less test setups:
-  # tstsetups <- grep("\\.glm\\.", names(vss), value = TRUE)
-  ###
   for (tstsetup in tstsetups) {
     args_vs_i <- args_vs[[tstsetup]]
     tstsetup_ref <- args_vs_i$tstsetup_ref
     pkg_crr <- args_vs_i$pkg_nm
     mod_crr <- args_vs_i$mod_nm
     fam_crr <- args_vs_i$fam_nm
-    refmod_crr <- refmods[[tstsetup_ref]]
+    if (!all(refmods[[tstsetup_ref]]$offset == 0)) {
+      offs_crr <- offs_tst
+    } else {
+      offs_crr <- rep(0, nobsv)
+    }
+    if (!all(refmods[[tstsetup_ref]]$wobs == 1)) {
+      wobs_crr <- wobs_tst
+    } else {
+      wobs_crr <- rep(1, nobsv)
+    }
+    formul_fit_crr <- args_fit[[args_vs_i$tstsetup_fit]]$formula
+    dat_crr <- get_dat_formul(formul_crr = formul_fit_crr,
+                              needs_adj = grepl("\\.spclformul", tstsetup))
     d_test_crr <- list(
-      y = refmod_crr$y,
-      test_points = seq_along(refmod_crr$y),
-      data = refmod_crr$fetch_data(),
-      weights = refmod_crr$wobs,
-      type = "test",
-      offset = refmod_crr$offset
+      data = dat,
+      offset = offs_crr,
+      weights = wobs_crr,
+      y = dat_crr[[stdize_lhs(formul_fit_crr)$y_nm]]
     )
     vs_repr <- do.call(varsel, c(
-      list(object = refmod_crr, d_test = d_test_crr),
+      list(object = refmods[[tstsetup_ref]], d_test = d_test_crr),
       excl_nonargs(args_vs_i)
     ))
     meth_exp_crr <- args_vs_i$method
@@ -104,8 +115,8 @@ test_that("`d_test` works", {
     }
     vsel_tester(
       vs_repr,
-      refmod_expected = refmod_crr,
-      dtest_expected = d_test_crr,
+      refmod_expected = refmods[[tstsetup_ref]],
+      dtest_expected = c(list(type = "test"), d_test_crr),
       solterms_len_expected = args_vs_i$nterms_max,
       method_expected = meth_exp_crr,
       nprjdraws_search_expected = args_vs_i$nclusters,
@@ -115,14 +126,163 @@ test_that("`d_test` works", {
         all(grepl("\\+", args_vs_i$search_terms)),
       info_str = tstsetup
     )
-    expect_identical(vs_repr$d_test, d_test_crr, info = tstsetup)
-    expect_equal(vs_repr[setdiff(names(vs_repr), vsel_nms_dtest)],
-                 vss[[tstsetup]][setdiff(names(vss[[tstsetup]]),
-                                         vsel_nms_dtest)],
+    expect_equal(vs_repr[setdiff(names(vs_repr), "d_test")],
+                 vss[[tstsetup]][setdiff(names(vss[[tstsetup]]), "d_test")],
+                 info = tstsetup)
+    expect_equal(vs_repr$d_test[setdiff(names(vs_repr$d_test),
+                                        c("type", "data"))],
+                 vss[[tstsetup]]$d_test[setdiff(names(vss[[tstsetup]]$d_test),
+                                                c("type", "data"))],
                  info = tstsetup)
   }
 })
 
+test_that(paste(
+  "`d_test` set to actual test data gives a `<vsel_object>$summaries$sub`",
+  "object that can be reproduced by proj_linpred() and a",
+  "`<vsel_object>$summaries$ref` object that can be reproduced by",
+  "posterior_epred() and log_lik()"
+), {
+  skip_if_not(run_vs)
+  if (exists(".Random.seed", envir = .GlobalEnv)) {
+    rng_old <- get(".Random.seed", envir = .GlobalEnv)
+  }
+  tstsetups <- names(vss)
+  ### TODO (GAMMs): Currently, the following test setups (can) lead to the error
+  ### ```
+  ### Error in t(as.matrix(b$reTrms$Zt[ii, ])) %*%
+  ### as.matrix(c(as.matrix(ranef[[i]]))) :
+  ###   non-conformable arguments
+  ### ```
+  ### thrown by predict.gamm4(). This needs to be fixed. For now, exclude these
+  ### test setups:
+  tstsetups <- grep("\\.gamm\\.", tstsetups, value = TRUE, invert = TRUE)
+  ###
+  for (tstsetup in tstsetups) {
+    args_vs_i <- args_vs[[tstsetup]]
+    tstsetup_ref <- args_vs_i$tstsetup_ref
+    pkg_crr <- args_vs_i$pkg_nm
+    mod_crr <- args_vs_i$mod_nm
+    fam_crr <- args_vs_i$fam_nm
+    if (!all(refmods[[tstsetup_ref]]$offset == 0)) {
+      offs_crr <- offs_indep
+    } else {
+      offs_crr <- rep(0, nobsv_indep)
+    }
+    if (!all(refmods[[tstsetup_ref]]$wobs == 1)) {
+      wobs_crr <- wobs_indep
+    } else {
+      wobs_crr <- rep(1, nobsv_indep)
+    }
+    formul_fit_crr <- args_fit[[args_vs_i$tstsetup_fit]]$formula
+    dat_indep_crr <- get_dat_formul(
+      formul_crr = formul_fit_crr,
+      needs_adj = grepl("\\.spclformul", tstsetup),
+      dat_crr = dat_indep
+    )
+    d_test_crr <- list(
+      data = dat_indep,
+      offset = offs_crr,
+      weights = wobs_crr,
+      y = dat_indep_crr[[stdize_lhs(formul_fit_crr)$y_nm]]
+    )
+    vs_indep <- do.call(varsel, c(
+      list(object = refmods[[tstsetup_ref]], d_test = d_test_crr),
+      excl_nonargs(args_vs_i)
+    ))
+    meth_exp_crr <- args_vs_i$method
+    if (is.null(meth_exp_crr)) {
+      meth_exp_crr <- ifelse(mod_crr == "glm", "L1", "forward")
+    }
+    vsel_tester(
+      vs_indep,
+      refmod_expected = refmods[[tstsetup_ref]],
+      dtest_expected = c(list(type = "test"), d_test_crr),
+      solterms_len_expected = args_vs_i$nterms_max,
+      method_expected = meth_exp_crr,
+      nprjdraws_search_expected = args_vs_i$nclusters,
+      nprjdraws_eval_expected = args_vs_i$nclusters_pred,
+      search_trms_empty_size =
+        length(args_vs_i$search_terms) &&
+        all(grepl("\\+", args_vs_i$search_terms)),
+      info_str = tstsetup
+    )
+
+    ### Summaries for the submodels -------------------------------------------
+
+    # For getting the correct seed in proj_linpred():
+    set.seed(args_vs_i$seed)
+    p_sel_dummy <- .get_refdist(refmods[[tstsetup_ref]],
+                                nclusters = vs_indep$nprjdraws_search)
+    # As soon as GitHub issues #168 and #211 are fixed, we can use `refit_prj =
+    # FALSE` here:
+    pl_indep <- proj_linpred(vs_indep,
+                             newdata = dat_indep_crr,
+                             offsetnew = d_test_crr$offset,
+                             weightsnew = d_test_crr$weights,
+                             transform = TRUE,
+                             integrated = TRUE,
+                             .seed = NA,
+                             nterms = c(0L, seq_along(vs_indep$solution_terms)),
+                             nclusters = args_vs_i$nclusters_pred,
+                             seed = NA)
+    summ_sub_ch <- lapply(pl_indep, function(pl_indep_k) {
+      names(pl_indep_k)[names(pl_indep_k) == "pred"] <- "mu"
+      names(pl_indep_k)[names(pl_indep_k) == "lpd"] <- "lppd"
+      pl_indep_k$mu <- unname(drop(pl_indep_k$mu))
+      pl_indep_k$lppd <- drop(pl_indep_k$lppd)
+      return(pl_indep_k)
+    })
+    names(summ_sub_ch) <- NULL
+    expect_equal(vs_indep$summaries$sub, summ_sub_ch,
+                 tolerance = .Machine$double.eps, info = tstsetup)
+
+    ### Summaries for the reference model -------------------------------------
+
+    if (pkg_crr == "rstanarm") {
+      mu_new <- rstantools::posterior_epred(refmods[[tstsetup_ref]]$fit,
+                                            newdata = dat_indep,
+                                            offset = d_test_crr$offset)
+      if (grepl("\\.without_wobs", tstsetup)) {
+        lppd_new <- rstantools::log_lik(refmods[[tstsetup_ref]]$fit,
+                                        newdata = dat_indep,
+                                        offset = d_test_crr$offset)
+      } else {
+        ### Currently, rstanarm issue #567 causes an error to be thrown when
+        ### calling log_lik(). Therefore, use the following dummy which
+        ### guarantees test success:
+        lppd_new <- matrix(vs_indep$summaries$ref$lppd,
+                           nrow = nrefdraws, ncol = nobsv_indep, byrow = TRUE)
+        ###
+      }
+    } else if (pkg_crr == "brms") {
+      mu_new <- rstantools::posterior_epred(refmods[[tstsetup_ref]]$fit,
+                                            newdata = dat_indep)
+      if (fam_crr == "binom") {
+        # Compared to rstanarm, brms uses a different convention for the
+        # binomial family: The values returned by posterior_epred() are not
+        # probabilities, but the expected values on the scale of the response
+        # (so the probabilities multiplied by the number of trials). Thus, we
+        # have to revert this here:
+        mu_new <- mu_new / matrix(wobs_indep, nrow = nrow(mu_new),
+                                  ncol = ncol(mu_new), byrow = TRUE)
+      }
+      lppd_new <- rstantools::log_lik(refmods[[tstsetup_ref]]$fit,
+                                      newdata = dat_indep)
+    }
+    summ_ref_ch <- list(
+      mu = unname(colMeans(mu_new)),
+      lppd = unname(apply(lppd_new, 2, log_sum_exp) - log(nrefdraws))
+    )
+    expect_equal(vs_indep$summaries$ref, summ_ref_ch,
+                 tolerance = 1e2 * .Machine$double.eps, info = tstsetup)
+    lppd_ref_ch2 <- unname(loo::elpd(lppd_new)$pointwise[, "elpd"])
+    expect_equal(vs_indep$summaries$ref$lppd, lppd_ref_ch2,
+                 tolerance = 1e2 * .Machine$double.eps, info = tstsetup)
+  }
+  if (exists("rng_old")) assign(".Random.seed", rng_old, envir = .GlobalEnv)
+})
+
 ## Regularization ---------------------------------------------------------
 
 # In fact, `regul` is already checked in `test_project.R`, so the `regul` tests
@@ -420,7 +580,7 @@ test_that("for L1 search, `penalty` has an expected effect", {
   }
 })
 
-# search_terms ------------------------------------------------------------
+## search_terms -----------------------------------------------------------
 
 test_that(paste(
   "including all terms in `search_terms` gives the same results as the default",
@@ -563,6 +723,8 @@ test_that("`seed` works (and restores the RNG state afterwards)", {
   }
 })
 
+## nloo -------------------------------------------------------------------
+
 test_that("invalid `nloo` fails", {
   for (tstsetup in names(refmods)) {
     # Use suppressWarnings() because of occasional warnings concerning Pareto k
@@ -646,6 +808,8 @@ test_that("setting `nloo` smaller than the number of observations works", {
   }
 })
 
+## validate_search --------------------------------------------------------
+
 test_that("`validate_search` works", {
   skip_if_not(run_cvvs)
   tstsetups <- grep("\\.default_cvmeth", names(cvvss), value = TRUE)
@@ -741,6 +905,8 @@ test_that("`validate_search` works", {
   expect_true(sum(!suggsize_cond, na.rm = TRUE) <= sum_as_unexpected)
 })
 
+## Arguments specific to K-fold CV ----------------------------------------
+
 test_that("invalid `K` fails", {
   expect_error(cv_varsel(refmods[[1]], cv_method = "kfold", K = 1),
                "^`K` must be at least 2\\.$")
@@ -779,12 +945,11 @@ test_that(paste(
     fit_crr <- fits[[tstsetup_fit]]
     K_crr <- args_cvvs_i$K
 
-    # Refit `K_crr` times:
-    if (grepl("\\.glmm\\.", tstsetup) && FALSE) {
-      # TODO: Currently, this case is deactivated because of an error thrown by
-      # lme4. When resolved, activate it by removing the ` && FALSE` part inside
-      # of the `if` condition.
-
+    # Refit `K_crr` times (note: below, the seed for constructing `folds_vec`
+    # had to be changed in some cases to avoid unfavorable PRNG situations,
+    # leading to technical issues such as nonconvergence of the submodel fitter;
+    # this is also tied to the value of `seed_tst`):
+    if (grepl("\\.glmm\\.", tstsetup)) {
       # Perform a grouped K-fold CV to test an edge case where all observations
       # belonging to the same level of a variable with group-level effects are
       # in the same fold, so prediction is performed for new levels (see, e.g.,
@@ -792,13 +957,12 @@ test_that(paste(
       if (exists(".Random.seed", envir = .GlobalEnv)) {
         rng_old <- get(".Random.seed", envir = .GlobalEnv)
       }
-      set.seed(seed2_tst) # Makes the construction of the CV folds reproducible.
+      # Make the construction of the CV folds reproducible:
+      set.seed(seed2_tst * 3L)
       folds_vec <- loo::kfold_split_grouped(K = K_crr, x = dat$z.1)
       if (exists("rng_old")) assign(".Random.seed", rng_old, envir = .GlobalEnv)
     } else {
-      # Change seed to avoid unfavorable CV folds causing an `nAGQ` error by
-      # lme4 during projection:
-      folds_vec <- cvfolds(nobsv, K = K_crr, seed = seed2_tst + 1L)
+      folds_vec <- cvfolds(nobsv, K = K_crr, seed = seed2_tst)
     }
     # Additionally to suppressWarnings(), suppressMessages() could be used here
     # (but is not necessary since messages seem to be suppressed within
@@ -886,7 +1050,10 @@ test_that(paste(
     fit_crr <- fits[[tstsetup_fit]]
     K_crr <- args_cvvs_i$K
 
-    # Refit `K_crr` times:
+    # Refit `K_crr` times (note: below, the seed for constructing `folds_vec`
+    # had to be changed in some cases to avoid unfavorable PRNG situations,
+    # leading to technical issues such as nonconvergence of the submodel fitter;
+    # this is also tied to the value of `seed_tst`):
     if (grepl("\\.glmm\\.", tstsetup)) {
       # Perform a grouped K-fold CV to test an edge case where all observations
       # belonging to the same level of a variable with group-level effects are
@@ -895,9 +1062,12 @@ test_that(paste(
       if (exists(".Random.seed", envir = .GlobalEnv)) {
         rng_old <- get(".Random.seed", envir = .GlobalEnv)
       }
-      set.seed(seed2_tst) # Makes the construction of the CV folds reproducible.
+      # Make the construction of the CV folds reproducible:
+      set.seed(seed2_tst + 10L)
       folds_vec <- loo::kfold_split_grouped(K = K_crr, x = dat$z.1)
       if (exists("rng_old")) assign(".Random.seed", rng_old, envir = .GlobalEnv)
+    } else if (grepl("\\.gam\\.", tstsetup)) {
+      folds_vec <- cvfolds(nobsv, K = K_crr, seed = seed2_tst + 10L)
     } else {
       folds_vec <- cvfolds(nobsv, K = K_crr, seed = seed2_tst)
     }
